procedure eso2fits (image)

string	image	=""			{prompt="Image name"}
string	transtab="rjlib$esohdr.trans"	{prompt="Keyword translation table"}
string	unident	="quit"		{enum="quit|warn",
				 prompt="Warn or quit for unknown keyword?"}
bool	savehdr	=yes		{prompt="Save original image header?"}
bool	verbose	=no		{prompt="Verbose messages?"}

# @(#) task eso2fits                   Author: R.A. Jansen -- Dec 2 2001
# @(#)
# @(#) task to convert image headers in ESO Hierarchical Format to stan-
# @(#) dard FITS headers using a keyword translation look-up table. That
# @(#) table lists one line per header keyword  and consists of 3 fields
# @(#) separated by a '!'.  The first field is the full ESO Hierarchical
# @(#) Format keystring, the second field is a standard 8-character FITS
# @(#) keyword  that is (defined to be) equivalent in meaning to the ESO
# @(#) keystring, and the third field defined a comment field (which may
# @(#) blank).  If no translation is found in the table, the task either
# @(#) quits or issues a warning,  depending on the value of  'unident'.
# @(#) In the case of a warning, the original ESO Hierarchival keystring
# @(#) is copied to the output header.
# @(#) The default translation table is valid for data obtained with ESO
# @(#) VLT's FORS instument in direct imaging, MOS and longslit mode but
# @(#) may be copied and adapted to a user's need.
# @(#) This task is more flexible (if slower) than  'esowfi.esohdr'  for
# @(#) simple FITS images and allows the user to define their own trans-
# @(#) lations that may, e.g., retain information from the third word of
# @(#) the ESO keyword string. It may also be run with  'unident="warn"'
# @(#) prior to running  'esowfi.esohdr'  to only translate  a subset of
# @(#) the header keystrings differently than 'esohdr' normally would.
# @(#) Full support for multi-extension FITS files will be considered in
# @(#) the near future but is not available in the present version.
# @(#)
# @(#) Uses 'rjtools' tasks:  chkimg, gdate
# @(#)
# @(#) Aug  5 2003 -- Cleaned up code; added 'savehdr'; wrote help. RAJ.

begin
	string	cimg, cimr, ctab, ctyp, ctmp, cdir
	int	nl, nw, nc
	bool	sh, vb, quit
	
#	Initialize variables...
	cimg	= image
	ctab	= transtab
	quit	= yes
	sh	= savehdr
	vb	= verbose
	if ( unident == "warn" ) { quit = no }

#	Load required package if not loaded already...
	if ( !defpac("artdata") ) { artdata }

#	Determine task directory (for 'csh' script)...
	path("rjtools$esohdrs/") | translit("-", "!", " ", del-, col-) | \
		scan (ctmp, cdir)

#	Check whether image exists...
	chkimg (cimg, "access", imtype="", verbose+)
	if (!chkimg.ok) { return }
	cimr = chkimg.impath//chkimg.imroot
	ctyp = chkimg.imtype
	cimg = cimr//ctyp

	if ( chkimg.imrpstr != "" ) {
	   # The special case of extension "[0]" is dealt with sort of ok
	   # thanks to inheritance, but may result in two copies of the
	   # header (primary header and secondary header corresponding to
	   # the one and only data unit); proceed with caution...
	   if ( chkimg.imrpstr == "[0]" ) {
	      cimg = cimg//"[0]"
	   } else {
	      print ("SORRY: no support yet for multi-extension FITS files!")
	      if (vb) print ("       Try complaining to the author and CC his boss..")
	      return
	   }
	}

#	Check whether the image header has been translated already...
	hselect (cimg, "eso2fit*", "yes") | count | scan (nl, nw, nc)
	if ( nw > 0 ) {
	   printf("WARNING: ESO2FITS was run on this image already! Proceed anyway? [y/n] (y):")
	   scan (ctmp)
	   print (ctmp) | lcase | scan(ctmp)
	   if ( ctmp=="n" || ctmp=="no" ) { return }
	}

#	Check whether header translation table exists...
	chkimg (ctab, "access", imtype="file", verbose+)
	if (!chkimg.ok) { return }
	ctab = chkimg.impath//chkimg.imroot

#	Dump the original image header...
	chkimg (cimr//".hdr", "noaccess", imtype="file", verbose-)
	if (!chkimg.ok) { 
	   chkimg (cimr//".hdr~", "noaccess", imtype="file", verbose-)
	   if (!chkimg.ok) {
	      printf("Found \""//cimr//".hdr\" and \""//cimr//".hdr~\". Shall I delete \""//cimr//".hdr~\"? [y/n] (n):")
	      scan (ctmp)
	      print (ctmp) | lcase | scan(ctmp)
	      if ( ctmp!="y" && ctmp!="yes" ) { return }
	      delete (cimr//".hdr~", yes, verify-)
	   }
	   rename (cimr//".hdr", cimr//".hdr~", field="all")
	}
	chkimg (cimr//".tmp", "noaccess", imtype="file", verbose-)
	if (!chkimg.ok) { delete (cimr//".tmp", yes, verify-) }
	imheader (cimg, longheader+, > cimr//".tmp")
	type (cimr//".tmp") | match ("ESO2FITS", "", stop+, > cimr//".hdr")

#	Prepare matching script...
	chkimg ("_tmps2fts.cl", "noaccess", imtype="file", verbose-)
	if (!chkimg.ok) { delete ("_tmps2fts.cl", yes, verify-) }
	ctmp = "!"//cdir//"eso2fits.csh "//cimr//".hdr "//ctab
	if (quit) { ctmp = ctmp//" -q" }
	if (vb)   { ctmp = ctmp//" -v" }
	print (ctmp, > "_tmps2fts.cl")

#	Call csh-script to do the actual matching...
	if (vb) { print ("Calling script "//cdir//"eso2fits.csh ...") }
	cl < _tmps2fts.cl

#	Add header keyword 'eso2fits'...
	gdate()
	printf("%8s= '%19s' / Conversion of ESO Hierarchical header\n", \
		"ESO2FITS", gdate.fdate, >> cimr//".hdr")

#	Replace the original image header by the translated one...
	mkheader (cimg, cimr//".hdr", append=no, verbose=vb)

#	Clean-up...
	delete ("_tmps2fts.cl,"//cimr//".hdr", yes, verify-)
	if (sh) {
	   copy (cimr//".tmp", cimr//".hdr", verbose-)
	   print ("Original ESO Hierarchical header is saved in \""//cimr//".hdr\".")
	}
	delete (cimr//".tmp", yes, verify-)

end
