procedure rpparse(imrpstr)

string	imrpstr	=""		{prompt="image region/plane string"}
bool	ok	=no		{prompt="is string a valid section/plane?"}
int	xlo	=INDEF		{prompt="x_start"}
int	xhi	=INDEF		{prompt="x_end"}
int	ylo	=INDEF		{prompt="y_start"}
int	yhi	=INDEF		{prompt="y_end"}
real	xcen	=INDEF		{prompt="x_center"}
real	ycen	=INDEF		{prompt="y_center"}
int	pln	=0		{prompt="image plane/extension"}
bool	verbose	=yes		{prompt="verbose error message?"}

# @(#) task rpparse                Author: R.A. Jansen -- August 25 2000
# @(#) 
# @(#) task to test  and parse  an image region/plane string  of generic
# @(#) format [pln][xlo:xhi,ylo:yhi], where the image plane is optional.
# @(#) Upper and lower pixel coordinate limits (xlo, xhi, ylo, yhi),  as
# @(#) well as coordinates of the centers in each dimension (xcen, ycen)
# @(#) are returned as task parameters,  as is the  plane/extension.  If
# @(#) wild-cards are present  in the input region string,  the returned
# @(#) coordinates will be INDEF.
# @(#) 
# @(#) Examples of valid plane/extension specifiers are:
# @(#)   [0], [1], ... , [256]
# @(#) Examples of valid regions (2-D image sections) are:
# @(#)   [*,*], [*,1:256], [1:256,*], [256:767,1:1024]
# @(#)
# @(#) Note that *implicit* 1-D image sections like "[1,1:256]"  are not
# @(#) allowed  by this task,  and must be replaced  by their *explicit*
# @(#) equivalent: "[1:1,1:256]". Also, wildcards are not allowed in the
# @(#) image plane/extension specifier.
# @(#)
# @(#) Uses rjtools task:  cparse
# @(#)
# @(#) This task  generalizes  task  'gprst48'  in the  'ccd48' package.
# @(#) Jul 18 2002 -- # of image planes no longer restricted to 7.  RAJ.
# @(#) Aug  4 2003 -- Complete rewrite  of this task;  added support for
# @(#)                wild-cards;  256 extensions are now allowed.  RAJ.

begin
	string	cstr, csav, cpln, ctmp
	int	i, j, k, m, n, x1, x2, y1, y2, llen
	bool	vb, bok, wld

#	Initialize variables...
	cstr = imrpstr
	cpln = ""
	bok  = no
	wld  = no
	llen = strlen(cstr)

#	Re-initialize task parameters...
	imrpstr = cstr
	vb	= verbose
	ok	= no
	xlo	= INDEF
	xhi	= INDEF
	ylo	= INDEF
	yhi	= INDEF
	xcen	= INDEF
	ycen	= INDEF
	pln	= 0

#	Check whether a string was given. Note that the null string is a
#	valid regions/plane string since it denotes the whole image ...
	if ( llen == 0 ) {
	   ok = yes
	   return
	}

#	NOTE: although I'd rather use 'error' rather than 'print' followed
#	by 'return', the result of 'error' is an abort of the task WITHOUT
#	updating the task parameter file in uparm$.  Since we need to rely
#	on the task parameters being meaningful no matter what, I have to
#	take the 'print(message)' + 'return' route...

#	Check whether 'imrpstr' conforms to a valid format...
	# image sections must start with a '['...
	i = stridx("[",cstr)
	if ( i != 1 ) {
	   if (vb) print ("ERROR: \""//cstr//"\" must start with a \"[\"!")
	   return
	}
	# and they must end with a ']'...
	if ( substr(cstr,llen,llen) != ']' ) {
	   if (vb) print ("ERROR: \""//cstr//"\" must end with a \"[\"!")
	   return
	}

	# there may be at most two pairs of brackets present...
	cparse (cstr, delim="[", nselect=0, lignore+, mignore-, verbose-)
	i = cparse.nfields
	if ( i > 2 ) {
	   if (vb) print ("ERROR: \""//cstr//"\" contains more than two \"[\"'s!")
	   return
	}
	cparse (cstr, delim="]", nselect=1, lignore+, mignore-, verbose-)
	j = cparse.nfields
	if ( j > 2 ) {
	   if (vb) print("ERROR: \""//cstr//"\" contains more than two \"]\"'s!")
	   return
	}
	# and the number of "]" needs to equal the number of "["...
	if ( j != i ) {
	   if (vb) print ("ERROR: number of \"[\" and \"]\" in \""//cstr//"\" differ!")
	   return
	}

	# Exhaustively check all possibilities -- resulting in the following
	# somewhat obtuse code.  There are some extra commented calls to 
	# 'cparse' that hopefully help keep track of what substring of the
	# region/plane string is being evaluated ...

	if ( i == 1 ) {
#1	   # there is one pair of brackets, check whether it specifies an
	   # image section or an image plane (extension)...
	   #cparse (cstr, delim="]", nselect=1, lignore+, mignore-, verbose-)
	   csav = substr(cparse.cfield,2,strlen(cparse.cfield))
	   if ( csav == "" ) {
	      if (vb) print ("ERROR: unspecified (empty) image section \"[]\"!")
	      return
	   }
	   cparse (csav, delim=",", nselect=1, lignore-, mignore-, verbose-)
	   ctmp = cparse.cfield
	   j = cparse.nfields
	   if ( j == 1 ) {
	      cparse (csav, delim=":", nselect=0, lignore-, mignore-, verbose-)
	      if ( cparse.nfields > 1 ) {
		 if (vb) print ("ERROR: unsupported 1-D section or switched comma and colon: \"["//csav//"]\"")
	         return
	      } else {
	         # Not a region/section string. Verify that it is a valid
	         # plane/extension identifier...
	         m = 10**(strlen(ctmp)-1)
	         n = 10**(strlen(ctmp)) - 1
	         if ( m == 1 ) { m = 0 }
	         if ( n > 256 ) { n = 256 }
	         for ( k = m ; k <= n ; k = k + 1 ) {
		    if ( csav == str(k) ) {
		       bok  = yes
		       cpln = str(k)
		       break
		    }
	         }
	      }
	      if ( cpln == "" ) {
		 if (vb) print ("ERROR: image extension must be an integer in range 0..256!")
		 return
	      }
	   } else if ( j == 2 ) {
	      # Not a plane/extension identifier. Verify that it is a valid
	      # region/section string...
	      # Check for wild-card specifier...
	      if ( ctmp == "*" ) {
		 wld = yes
	      } else {
		 # Not a wild-card; check for "xlo:xhi" range specifier...
		 cparse (ctmp, delim=":", nselect=1, ligno-, migno-, verbose-)
		 if ( cparse.nfields != 2) {
		    if (vb) print ("ERROR: invalid x-range specifier \"["//ctmp//"\": missing colon!")
		    return
		 } else if ( strlen(cparse.cfield) == 0 ) {
		    if (vb) print ("ERROR: invalid lower x-range limit \"\"!")
		    return
		 } else {
		    printf("%d\n", cparse.cfield) | scan(x1)
		    if ( cparse.cfield != str(x1) ) {
		       if (vb) print ("ERROR: invalid lower x-range limit \""//cparse.cfield//"\"!")
		       return
		    } else if ( x1 < 0 ) {
		       if (vb) print ("ERROR: invalid lower x-range limit "//str(x1)//" < 0!")
		       return
		    } else { xlo = x1 }
		    cparse (ctmp, delim=":", nselect=2, ligno-, migno-, verb-)
		    if ( strlen(cparse.cfield) == 0 ) {
		       if (vb) print ("ERROR: invalid upper x-range limit \"\"!")
		       return
		    } else {
		       printf("%d\n", cparse.cfield) | scan(x2)
		       if ( cparse.cfield != str(x2) ) {
			  if (vb) print ("ERROR: invalid upper x-range limit \""//cparse.cfield//"\"!")
			  return
		       } else if ( x2 < 0 ) {
		          if (vb) print ("ERROR: invalid upper x-range limit "//str(x2)//" < 0!")
			  return
		       } else if ( x2 < x1 ) {
			  if (vb) print ("ERROR: invalid x-range: xhi="//str(x2)//" < "//str(x1)//"=xlo!")
			  return
		       } else { xhi = x2 }
		    }
		 }
	      }
	      if (!wld) {
		 xcen = 0.5*(x1+x2)
	      }

	      cparse (csav, delim=",", nselect=2, lignore-, mignore-, verbose-)
	      ctmp = cparse.cfield
	      # Check for wild-card specifier...
	      wld = no
	      if ( ctmp == "*" ) {
		 wld = yes
	      } else {
		 # Not a wild-card; check for "ylo:yhi" range specifier...
		 cparse (ctmp, delim=":", nselect=1, ligno-, migno-, verbose-)
		 if ( cparse.nfields != 2) {
		    if (vb) print ("ERROR: invalid y-range specifier \""//ctmp//"\": missing colon!")
		    xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		    return
		 } else if ( strlen(cparse.cfield) == 0 ) {
		    if (vb) print ("ERROR: invalid lower y-range limit \"\"!")
		    xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		    return
		 } else {
		    printf("%d\n", cparse.cfield) | scan(y1)
		    if ( cparse.cfield != str(y1) ) {
		       if (vb) print ("ERROR: invalid lower y-range limit \""//cparse.cfield//"\"!")
		       xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		       return
		    } else if ( y1 < 0 ) {
		       if (vb) print ("ERROR: invalid lower y-range limit "//str(y1)//" < 0!")
		       xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		       return
		    } else { ylo = y1 }
		    cparse (ctmp, delim=":", nselect=2, ligno-, migno-, verb-)
		    if ( strlen(cparse.cfield) == 0 ) {
		       if (vb) print ("ERROR: invalid upper y-range limit \"\"!")
		       xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		       return
		    } else {
		       printf("%d\n", cparse.cfield) | scan(y2)
		       if ( cparse.cfield != str(y2) ) {
			  if (vb) print ("ERROR: invalid upper y-range limit \""//cparse.cfield//"\"!")
			  xlo = INDEF ; xhi = INDEF ; xcen = INDEF
			  return
		       } else if ( y2 < 0 ) {
		          if (vb) print ("ERROR: invalid upper y-range limit "//str(x2)//" < 0!")
			  xlo = INDEF ; xhi = INDEF ; xcen = INDEF
			  return
		       } else if ( y2 < y1 ) {
			  if (vb) print ("ERROR: invalid y-range: yhi="//str(y2)//" < "//str(y1)//"=ylo!")
			  xlo = INDEF ; xhi = INDEF ; xcen = INDEF
			  return
		       } else { yhi = y2 }
		    }
		 }
	      }
	      if (!wld) {
		 ycen = 0.5*(y1+y2)
	      }

	   } else if ( j == 3 ) {
	      cparse (csav, delim=":", nselect=0, lignore-, mignore-, verbose-)
	      if ( cparse.nfields > 1 ) {
	         if (vb) print ("ERROR: unsupported 3-D section or switched comma and colon: \"["//csav//"]\"")
	         return
	      } else {
	         if (vb) print ("ERROR: invalid or unsupported image section \"["//csav//"]\"!")
	         return
	      }

	   } else {
	      if (vb) print ("ERROR: invalid image section \"["//csav//"]\"!")
	      return
	   }

	   bok = yes

	} else {
#2	   # there are two pairs of brackets; the first presumably a plane
	   # specifier (only valid option) ...
	   cparse (cstr, delim="]", nselect=1, lignore+, mignore-, verbose-)
	   csav = substr(cparse.cfield,2,strlen(cparse.cfield))
	   m = 10**(strlen(csav)-1)
	   n = 10**(strlen(csav)) - 1
	   if ( m == 1 ) { m = 0 }
	   if ( n > 256 ) { n = 256 }
	   for ( k = m ; k <= n ; k = k + 1 ) { 
	      if ( csav == str(k) ) {
		 bok  = yes
		 cpln = str(k)
		 break
	      }
	   }
	   if ( cpln == "" ) {
	      cparse (csav, delim=",", nselect=0, ligno-, migno-, verbose-)
	      if ( cparse.nfields > 1 ) {
		 if (vb) print ("ERROR: image extension specifier must precede section in \""//cstr//"\"!")
		 return
	      } else {
	         if (vb) print ("ERROR: image extension must be an integer in range 0..256!")
		 return
	      }
	   }

	   # The second pair presumably contains a region specfier...
	   cparse (cstr, delim="]", nselect=2, lignore+, mignore-, verbose-)
	   csav = substr(cparse.cfield,2,strlen(cparse.cfield))
	   if ( csav == "" ) {
	      if (vb) print ("ERROR: unspecified (empty) image section \"[]\"!")
	      return
	   }
	   cparse (csav, delim=",", nselect=1, lignore-, mignore-, verbose-)
	   ctmp = cparse.cfield
	   j = cparse.nfields
	   if ( j == 1 ) {
	      cparse (csav, delim=":", nselect=0, lignore-, mignore-, verbose-)
	      if ( cparse.nfields > 1 ) {
		 if (vb) print ("ERROR: unsupported 1-D section or switched comma and colon: \"["//csav//"]\"")
	         return
	      } else {
		 if (vb) print ("ERROR: invalid image section \"["//csav//"]\"!")
		 return
	      }
	   } else if ( j == 2 ) {
	      # Check for wild-card specifier...
	      if ( ctmp == "*" ) {
		 wld = yes
	      } else {
		 # Not a wild-card; check for "xlo:xhi" range specifier...
		 cparse (ctmp, delim=":", nselect=1, ligno-, migno-, verbose-)
		 if ( cparse.nfields != 2) {
		    if (vb) print ("ERROR: invalid x-range specifier \"["//ctmp//"\": missing colon!")
		    return
		 } else if ( strlen(cparse.cfield) == 0 ) {
		    if (vb) print ("ERROR: invalid lower x-range limit \"\"!")
		    return
		 } else {
		    printf("%d\n", cparse.cfield) | scan(x1)
		    if ( cparse.cfield != str(x1) ) {
		       if (vb) print ("ERROR: invalid lower x-range limit \""//cparse.cfield//"\"!")
		       return
		    } else if ( x1 < 0 ) {
		       if (vb) print ("ERROR: invalid lower x-range limit "//str(x1)//" < 0!")
		       return
		    } else { xlo = x1 }
		    cparse (ctmp, delim=":", nselect=2, ligno-, migno-, verb-)
		    if ( strlen(cparse.cfield) == 0 ) {
		       if (vb) print ("ERROR: invalid upper x-range limit \"\"!")
		       return
		    } else {
		       printf("%d\n", cparse.cfield) | scan(x2)
		       if ( cparse.cfield != str(x2) ) {
			  if (vb) print ("ERROR: invalid upper x-range limit \""//cparse.cfield//"\"!")
			  return
		       } else if ( x2 < 0 ) {
		          if (vb) print ("ERROR: invalid upper x-range limit "//str(x2)//" < 0!")
			  return
		       } else if ( x2 < x1 ) {
			  if (vb) print ("ERROR: invalid x-range: xhi="//str(x2)//" < "//str(x1)//"=xlo!")
			  return
		       } else { xhi = x2 }
		    }
		 }
	      }
	      if (!wld) {
		 xcen = 0.5*(x1+x2)
	      }

	      cparse (csav, delim=",", nselect=2, lignore-, mignore-, verbose-)
	      ctmp = cparse.cfield
	      # Check for wild-card specifier...
	      wld = no
	      if ( ctmp == "*" ) {
		 wld = yes
	      } else {
		 # Not a wild-card; check for "ylo:yhi" range specifier...
		 cparse (ctmp, delim=":", nselect=1, ligno-, migno-, verbose-)
		 if ( cparse.nfields != 2) {
		    if (vb) print ("ERROR: invalid y-range specifier \""//ctmp//"\": missing colon!")
		    xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		    return
		 } else if ( strlen(cparse.cfield) == 0 ) {
		    if (vb) print ("ERROR: invalid lower y-range limit \"\"!")
		    xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		    return
		 } else {
		    printf("%d\n", cparse.cfield) | scan(y1)
		    if ( cparse.cfield != str(y1) ) {
		       if (vb) print ("ERROR: invalid lower y-range limit \""//cparse.cfield//"\"!")
		       xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		       return
		    } else if ( y1 < 0 ) {
		       if (vb) print ("ERROR: invalid lower y-range limit "//str(y1)//" < 0!")
		       xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		       return
		    } else { ylo = y1 }
		    cparse (ctmp, delim=":", nselect=2, ligno-, migno-, verb-)
		    if ( strlen(cparse.cfield) == 0 ) {
		       if (vb) print ("ERROR: invalid upper y-range limit \"\"!")
		       xlo = INDEF ; xhi = INDEF ; xcen = INDEF
		       return
		    } else {
		       printf("%d\n", cparse.cfield) | scan(y2)
		       if ( cparse.cfield != str(y2) ) {
			  if (vb) print ("ERROR: invalid upper y-range limit \""//cparse.cfield//"\"!")
			  xlo = INDEF ; xhi = INDEF ; xcen = INDEF
			  return
		       } else if ( y2 < 0 ) {
		          if (vb) print ("ERROR: invalid upper y-range limit "//str(x2)//" < 0!")
			  xlo = INDEF ; xhi = INDEF ; xcen = INDEF
			  return
		       } else if ( y2 < y1 ) {
			  if (vb) print ("ERROR: invalid y-range: yhi="//str(y2)//" < "//str(y1)//"=ylo!")
			  xlo = INDEF ; xhi = INDEF ; xcen = INDEF
			  return
		       } else { yhi = y2 }
		    }
		 }
	      }
	      if (!wld) {
		 ycen = 0.5*(y1+y2)
	      }

	   } else if ( j == 3 ) {
	      cparse (csav, delim=":", nselect=0, lignore-, mignore-, verbose-)
	      if ( cparse.nfields > 1 ) {
	         if (vb) print ("ERROR: unsupported 3-D section or switched comma and colon: \"["//csav//"]\"")
	         return
	      } else {
	         if (vb) print ("ERROR: invalid or unsupported image section \"["//csav//"]\"!")
	         return
	      }

	   } else {
	      if (vb) print ("ERROR: invalid image section \"["//csav//"]\"!")
	      return
	   }

	   bok = yes

	}


#	Update task parameters...
	ok	= bok
	if ( cpln != "" ) { pln = int(cpln) }

end



# Tested using following region/plane identifiers:
#   [0], [1], [9], [10], [99], [100], [256]                        (ok=yes);
#   [257], [*], [a], [], 1, [2, 2]                                 (ok=no);
#   [1:256,256:512], [*,1:256], [1:256,*], [*,*]                   (ok=yes);
#   [1,256:256,512], [1,1:256], [1:256,1], [*:*], [*:], [:], [a:b] (ok=no);
#   [0][1:256,2:257], [2][1:256,2:257], [2][*,*], [2][*,1:256]     (ok=yes)
#   [2][a:b], [1][1,1:256], [a][1:256,*], [a][b;c]                 (ok=no).
