procedure getsky (image)

string	image	=""		{prompt="image name"}
int	maxiter	=10		{max=100,prompt="maximum number of iterations"}
real	lower	=INDEF		{prompt="lower cutoff for pixel values"}
real	upper	=INDEF		{prompt="upper cutoff for pixel values"}
real	lsigma	=5.		{prompt="lower sigma for pixel rejection"}
real	usigma	=5.		{prompt="upper sigma for pixel rejection"}
bool	shrink	=no		{prompt="shrink sigma rejection limits?"}
bool	border	=yes		{prompt="use border instead of full image?"}
string	bwidth	="0.1"		{prompt="width of border (fraction or # pix)"}
bool	show	=yes		{prompt="show the final result?"}
bool	verbose	=no		{prompt="print all results to screen?"}
bool	label	=yes		{prompt="label column headers?"}
int	siter	=INDEF		{prompt="number of rejection iterations"}
int	snpix	=INDEF		{prompt="number of pixels used for statistics"}
real	smean	=INDEF		{prompt="average sky background level"}
real	smidpt	=INDEF		{prompt="median sky background level"}
real	smode	=INDEF		{prompt="modal sky background level"}
real	srms	=INDEF		{prompt="rms of sky background level"}
real	smin	=INDEF		{prompt="minimum pixel value"}
real	smax	=INDEF		{prompt="maximum pixel value"}

# @(#) task getsky                   Author: R.A. Jansen -- Jan 29, 2002
# @(#)
# @(#) task to compute the aproximate (sky) background level in an image
# @(#) assuming that that background is fairly flat  (has no large scale
# @(#) gradient or prominent higher order structure), via iterative sig-
# @(#) ma clipping.  When either the statistics converge (no further are
# @(#) pixels rejected),  or the maximum number of iterations is reached
# @(#) the task stops.  Only pixels with values within the range defined
# @(#) by 'lower' and 'upper' are considered.
# @(#)
# @(#) Uses 'rjtools' task: 'chkimg', 'imgstat' and 'borderstat'
# @(#)
# @(#) Aug 15 2003 -- Added parameters 'border','bwidth'&'show'; changed
# @(#)                default values from 0. to INDEF;  cleaned-up  code
# @(#)                and wrote help file. RAJ.

begin
	string	cimg, cbwd
	real	rbd, rlo, rhi, slo, shi, tmp
	real	ravg, rmed, rmod, rsig, rmin, rmax, osig
	int	nmax, nc, nr, bd, ipix, opix, i
	bool	bsh

#	Initialize variables...
	cimg	= image
	nmax	= maxiter
	rlo	= lower
	rhi	= upper
	slo	= lsigma
	shi	= usigma
	bsh	= shrink
	cbwd	= bwidth
	rbd	= real(cbwd)

#	(Re)initialize task parameters...
	image	= cimg
	lower	= rlo
	upper	= rhi
	bwidth	= cbwd
	siter	= 0
	snpix	= 0
	smean	= INDEF
	smidpt	= INDEF
	smode	= INDEF
	srms	= INDEF
	smin	= INDEF
	smax	= INDEF

#       Check existence of input image...
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	cimg = chkimg.impath//chkimg.imroot//chkimg.imtype//chkimg.imrpstr

#	Opening message...
        if (verbose) print ("GETSKY: image=\""//cimg//"\"")

#       Read the image size...
	hselect (cimg, "i_naxis1", "yes") | scan (nc)
	hselect (cimg, "i_naxis2", "yes") | scan (nr)

#	Optionally, print column headers...
	if ( label && ( verbose || show ) ) {
	   printf("%4s %8s  %8s  %8s  %8s  %8s  %8s  %8s\n", "iter", "npix", 
		"mean", "midpt", "mode", "stddev", "min", "max")
	}

#	Measure raw image statistics...
	if ( border ) {
	   if ( rbd > 0.0 && rbd < 0.5 ) {
	      bd = int(rbd*0.5*(nc+nr))
	      if ( bd < 1 ) bd = 1
	   } else if ( rbd < 1.) {
	      print ("ERROR: 'bwidth' must be >= 1, or in range (0.0,0.5)!")
	      return
	   } else {
	      bd = int(rbd)
	   }
	   borderstat (cimg, bd, lower=rlo, upper=rhi, binwidth=0.1, verbose+,
			label-) | scan (ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	} else {
	   imgstat (cimg, lower=rlo, upper=rhi, binwidth=0.1, verbose+,
			label-) | scan (ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	}
	if (verbose) {
	   printf("%4d %8d  %8f  %8f  %8f  %8f  %8f  %8f\n", 0, ipix, ravg,
		rmed, rmod, rsig, rmin, rmax)
	}

	opix = ipix
	osig = rsig

	if ( rlo == INDEF ) rlo = rmed - slo*rsig
	if ( rhi == INDEF ) rhi = rmed + shi*rsig
	if ( (rmed-slo*rsig) > rlo ) rlo = rmed - slo*rsig
	if ( (rmed+shi*rsig) < rhi ) rhi = rmed + shi*rsig

#	Measure iterative image statistics...
	for ( i = 1 ; i <= nmax ; i = i + 1 ) {
	   if (border) {
	      borderstat (cimg, bd, lower=rlo, upper=rhi, binwidth=0.1, verb+,
			label-) | scan (ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	   } else {
	      imgstat (cimg, lower=rlo, upper=rhi, binwidth=0.1, verbose+,
			label-) | scan (ipix,ravg,rmed,rmod,rsig,rmin,rmax)
	   }
	   if ( ipix == opix || ipix > opix || rmed == INDEF || \
		rmod == INDEF || ( rsig > osig && osig > 0. ) ) {
	      if ( show && !verbose ) {
		 printf("%4d %8d  %8f  %8f  %8f  %8f  %8f  %8f\n", (i-1),
			ipix, ravg, rmed, rmod, rsig, rmin, rmax)
		 i = i - 1
	      }
	      break
	   } else if ( verbose || (i == nmax && show ) ) {
	      printf("%4d %8d  %8f  %8f  %8f  %8f  %8f  %8f\n", i,
			ipix, ravg, rmed, rmod, rsig, rmin, rmax)
	   }
	   if (bsh) {
	      tmp = rmed - (slo/log10(10+i))*rsig
	      if ( tmp > rlo ) rlo = tmp
	      tmp = rmed + (shi/log10(10+i))*rsig
	      if ( tmp < rhi ) rhi = tmp
	   } else {
	      if ( (rmed - slo*rsig) > rlo ) rlo = rmed - slo*rsig
	      if ( (rmed + shi*rsig) < rhi ) rhi = rmed + shi*rsig
	   }
	   opix = ipix
	   osig = rsig
	}

#	Check whether there where any pixels in the given range...
	if ( ipix == 0) {
	   printf("ERROR: No pixels with values between %g and %g!\n",rlo,rhi)
	   return
	}
	if ( ipix < 2 ) rsig = 0.

#       Return task parameters...
	image	= cimg
	maxiter	= nmax
	if (border) bwidth = str(bd)
	siter	= i
	snpix	= ipix
	smean	= ravg
	smidpt	= rmed
	smode	= rmod
	srms	= rsig
	smin	= rmin
	smax	= rmax

end
