procedure mkmosaic (image1, image2, output)

string	image1	=""		{prompt="name of input (reference) image 1"}
string	image2	=""		{prompt="name of input image 2"}
string	output	=""		{prompt="name of output mosaic image"}
string	orient	="right"	{prompt="place image2 to t|b|l|r of image1",
			 	 enum="top|bottom|left|right"}
int	margin	=4		{prompt="width of margin in pixels"}
real	bgvalue	=-100.		{prompt="background value in mosaic"}
real	scale	=1.		{prompt="multiply image2 by this factor"}
real	zero	=0.		{prompt="offset to add to pixel values image2"}
bool	border	=yes		{prompt="use image border for auto-scaling?"}
bool	verbose	=yes		{prompt="Verbose messages?"}

# @(#) task mkmosaic                  Author: R.A. Jansen -- May 10 1995
# @(#)
# @(#) task to create a mosaic  of two images of arbitrary dimension.  A
# @(#) gap of 'margin' pixels  with value 'bgvalue'  is left between the
# @(#) two images. The pixel values of 'image2' will be offset by 'zero'
# @(#) and multiplied by 'scale'.  'Image2' is placed to the top, bottom,
# @(#) left, or right of 'image1'.
# @(#)
# @(#) Uses 'rjtools' tasks: 'chkimg', 'rpparse', 'rpbuild'
# @(#)
# @(#) Jun  4 2002 -- replaced call to "ctio" task 'imcreate'  by block
# @(#)         replication of dev$pix.imh,  since that is guaranteed to
# @(#)         work for any IRAF installation. The header of the output
# @(#)         mosaic is copied from 'image1'. 'chkimg' is now used for
# @(#)         image checking. When 'scale=0.', auto-scaling is applied.
# @(#) Aug 15 2003 -- Added check for 'ctio' package: if defined,  then
# @(#)         used 'imcreate', otherwise block replicate dev$pix. RAJ.

begin
	string	img1, img2, out, ornt, csc1, csc2, sec1, sec2, sec3, szstr
	int	mrg, np, nax1, nax2, nax3, nax4, nax5, nax6, pln, ni, nj
	real	bg, scl, zr, sky1, sky2
	bool	bctio, bbrd

#	Initialize variables...
	img1  = image1
	img2  = image2
	out   = output
	ornt  = orient
	mrg   = margin
	bg    = bgvalue
	scl   = scale
	zr    = zero
	bbrd  = border
	bctio = ( envget("ctio") != "" )

	if ( ornt == "top" )    { ornt = "t" }
	if ( ornt == "bottom" ) { ornt = "b" }
	if ( ornt == "left" )   { ornt = "l" }
	if ( ornt == "right" )  { ornt = "r" }

#	Check that the 'stsdas' and 'artdata' packages are loaded...
	if ( !defpac("stsdas") ) { stsdas }
	if ( !defpac("artdata") ) { artdata }
	if ( bctio ) {
	   if ( !defpac("ctio") ) { ctio }
	}

#	Check existence of reference input image ...
	chkimg (img1, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	img1 = chkimg.impath//chkimg.imroot//chkimg.imtype
	csc1 = chkimg.imrpstr

	hselect (img1//csc1, "i_naxis1", "yes") | scan (nax1)
	hselect (img1//csc1, "i_naxis2", "yes") | scan (nax2)
	rpparse (chkimg.imrpstr, verbose-)
	pln = rpparse.pln
	rpbuild (1, nax1, 1, nax2, pln=pln)
	sec1 = rpbuild.imrpstr

#	Check existence of second input image ...
	chkimg (img2, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	img2 = chkimg.impath//chkimg.imroot//chkimg.imtype
	csc2 = chkimg.imrpstr

	hselect (img2//csc2, "i_naxis1", "yes") | scan (nax3)
	hselect (img2//csc2, "i_naxis2", "yes") | scan (nax4)
	rpparse (chkimg.imrpstr, verbose-)
	pln = rpparse.pln
	rpbuild (1, nax3, 1, nax4, pln=pln)
	sec2 = rpbuild.imrpstr

#	Check that the output image doesn't exist already ...
	chkimg (out, "noaccess", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	out = chkimg.impath//chkimg.imroot//chkimg.imtype
	if ( chkimg.imrpstr != "" ) {
	   print ("WARNING: Ignoring \""//chkimg.imrpstr//"\" for output mosaic.")
	}

#	Determine dimension of output mosaic image...
	if ( ornt == "t" ) {
	   nax5 = nax2 + mrg + 1
	   nax6 = nax2 + nax4 + mrg
	   if ( nax1 > nax3 ) { np = nax1 } else { np = nax3 }
	   szstr = "["//np//","//nax6//"]"
	} else if ( ornt == "b" ) {
	   nax5 = nax4 + mrg + 1
	   nax6 = nax4 + nax2 + mrg
	   if (nax3 > nax1 ) { np = nax3 } else { np = nax1 }
	   szstr = "["//np//","//nax6//"]"
	} else if ( ornt == "l" ) {
	   nax5 = nax3 + mrg + 1
	   nax6 = nax3 + nax1 + mrg
	   if ( nax2 > nax4 ) { np = nax2 } else { np = nax4 }
	   szstr = "["//nax6//","//np//"]"
	} else {
	   nax5 = nax1 + mrg + 1
	   nax6 = nax1 + nax3 + mrg
	   if ( nax4 > nax2 ) { np = nax4 } else { np = nax2 }
	   szstr = "["//nax6//","//np//"]"
	}

#	Create image template for new mosaic ...
	if (verbose) printf("\nCreating frame \""//out//szstr//"\"; pixel value = %7f\n", bg)
	if ( bctio ) {
	   if ( ornt == "t" || ornt == "b" ) {
	      imcreate (out, 2, np, nax6, INDEF, INDEF, INDEF, INDEF, INDEF,
			header="copy", pixtype="real", reference=img1)
	   } else {
	      imcreate (out, 2, nax6, np, INDEF, INDEF, INDEF, INDEF, INDEF,
			header="copy", pixtype="real", reference=img1)
	   }
	   imreplace (out, value=bg, imagina=0., lower=INDEF, upper=INDEF)
	} else {
	   if (access("tmpmkmsc.head")) delete ("tmpmkmsc.head", yes, verify-)
	   imcopy ("dev$pix.imh[1:2,1:2]", out, verbos-)
	   imheader (img1, longheader+, > "tmpmkmsc.head")
	   mkheader (out, "tmpmkmsc.head", append-, verbose-)
	   delete ("tmpmkmsc.head", yes, verify-)
	   imreplace (out, value=bg, imagina=0., lower=INDEF, upper=INDEF)
	   if ( ornt == "t" || ornt == "b" ) {
	      blkrep (out, out, int(0.5*np), int(0.5*nax6), 1, 1, 1, 1, 1)
	   } else {
	      blkrep (out, out, int(0.5*nax6), int(0.5*np), 1, 1, 1, 1, 1)
	   }
	}

#	Mosaic 'img1' and 'img2' into the output image ...
	if (verbose) print ("Mosaicing input frames ...")

	# Copy image1 (to support use of image sections)...
	imcopy (img1//csc1, "tmpmkmsc1.fits", verbose-)

	# Apply (optional) intensity scaling and offset...
	if ( scl == 0. ) {
	   getsky (img1//csc1, maxiter=5, lower=INDEF, upper=INDEF, lsigma=5.,
		usigma=5., shrink-, border=bbrd, bwidth="0.1", show+,
		verbose-, label-) | scan (ni, nj, sky1)
	   getsky (img2//csc2, maxiter=5, lower=INDEF, upper=INDEF, lsigma=5.,
		usigma=5., shrink-, border=bbrd, bwidth="0.1", show+,
		verbose-, label-) | scan (ni, nj, sky2)
	   scl = sky1/sky2
	}
	if (verbose) {
	   printf("Applying scaling and offset:  %8g*im2 + %.2f\n", scl, zr)
	}
	imcalc (img2//csc2, "tmpmkmsc2.fits", str(scl)//"*im1 + "//str(zr),
		nullval=0., verbose-)

#	Insert the two image into the mosaic image...
	if ( ornt == "t" ) {
	   if (verbose) print (img1//sec1," --> ",out//sec1)
	   imcopy ("tmpmkmsc1.fits"//sec1, out//sec1, verbose-)
	   sec3 = "[1:"//nax3//","//nax5//":"//nax6//"]"
	   if (verbose) print (img2//sec2," --> ",out//sec3)
	   imcopy ("tmpmkmsc2.fits"//sec2, out//sec3, verbose-)

	} else if ( ornt == "b" ) {
	   if (verbose) print (img2//sec2," --> ",out//sec2)
	   imcopy ("tmpmkmsc2.fits"//sec2, out//sec2, verbose-)
	   sec3 = "[1:"//nax1//","//nax5//":"//nax6//"]"
	   print (img1//sec1," --> ",out//sec3)
	   imcopy ("tmpmkmsc1.fits"//sec1, out//sec3, verbose-)

	} else if ( ornt == "r" ) {
	   if (verbose) print (img1//sec1," --> ",out//sec1)
	   imcopy ("tmpmkmsc1.fits"//sec1, out//sec1, verbose-)
	   sec3 = "["//nax5//":"//nax6//",1:"//nax4//"]"
	   print (img2//sec2," --> ",out//sec3)
	   imcopy ("tmpmkmsc2.fits"//sec2, out//sec3, verbose-)

	} else {
	   if (verbose) print (img2//sec2," --> ",out//sec2)
	   imcopy ("tmpmkmsc2.fits"//sec2, out//sec2, verbose-)
	   sec3 = "["//nax5//":"//nax6//",1:"//nax2//"]"
	   print ("tmpmkmsc1.fits"//sec1," --> ",out//sec3)
	   imcopy (img1//sec1, out//sec3, verbose-)

	}
	delete ("tmpmkmsc1.fits,tmpmkmsc2.fits", yes, verify-)

	if (verbose) print ("Done.")

end
