procedure ctrcoo (intab,outab,image)

string	intab	=""		{prompt="Input coordinate file"}
string	outab	=""		{prompt="Output coordinate file"}
string	image	=""		{prompt="Image containing the objects"}
int	cntrbox	=7		{prompt="Size of centering box in pixels"}
bool	redispl	=yes		{prompt="Redisplay the image first?"}
bool	verbose	=yes		{prompt="Print recentered coordinates?"}
real	xcen	=INDEF		{prompt="Fitted x-center position"}
real	ycen	=INDEF		{prompt="Fitted y-center position"}

# @(#) task ctrcoo                    Author: R.A. Jansen -- Jun 19 2000
# @(#)
# @(#) wrapper around 'imexamine' to (re)fit center positions to objects
# @(#) in an image, given initial estimates.  Only the first two columns
# @(#) in the input coordinate file are considered.  'outab' and 'intab'
# @(#) may be the same,  in which case  the input file  will be  updated
# @(#) (i.e. overwritten). If only one set of input coordinates is given
# @(#) upon exit of this task parameters 'xcen'  and 'ycen'  will return
# @(#) the fitted center; if more than one set of input coordinates is
# @(#) given, only the center coordinates for the last set is return.
# @(#)
# @(#) Calls 'rjtools' tasks: 'chkimg' and 'xdisplay'
# @(#)
# @(#) Aug 18 2003 -- Cleaned-up code (IRAF v2.12); wrote help file. RAJ

begin
	string	coo, ctr, cimg, tmpcoo, tmpctr
	real	x, y
	int	cbox, i

	# Initialize...
	coo	= intab
	ctr	= outab
	cimg	= image
	cbox	= 2*int(0.5*cntrbox)+1
	tmpcoo  = mktemp("tmpctrcoo")
	tmpctr	= mktemp("tmpctrctr")
	xcen	= INDEF
	ycen	= INDEF

	# Check whether the required packages are loaded...
	if ( !defpac("images") ) { images }
	if ( !defpac("tv") ) { tv }

	# Check whether the input file exists and the output file
	# doesn't (unless output file and input file are the same)...
	chkimg (coo, "access", imtype="file", verbose=yes)
	if (!chkimg.ok) { return }
	if ( ctr == coo ) {
	   coo = chkimg.impath//chkimg.imroot
	   ctr = coo
	} else {
	   chkimg (ctr, "noaccess", imtype="file", verbose=yes)
	   if (!chkimg.ok) { return }
	   ctr = chkimg.impath//chkimg.imroot
	}

	# Check whether the input image exists...
	chkimg (cimg, "access", imtype="", verbose=yes)
        if (!chkimg.ok) { return }
        cimg = chkimg.impath//chkimg.imroot//chkimg.imtype//chkimg.imrpstr

	# (Re-)display the image...
	if (redispl) {
	   xdisplay (cimg, buff=1, z1=INDEF, z2=INDEF, lsig=3., hsig=15.,
		niter=1, zmode="linear", verbose=no)
	}

	# Write a cursor file for 'imexam' ...
	if (verbose) { print ("Guessed center coordinates:") }
	list = coo
	i = 0
	while( fscan(list,x,y) != EOF ) {
           i += 1
	   print (x,y," 205 a", >> tmpcoo)
	   if (verbose) { printf("#%d  xtry=%.3f, ytry=%.3f\n", i, x, y) }
	}
	print ("1. 1. 205 q", >> tmpcoo)

	# Use 'imexam' to fit object center positions given the provided
	# estimated positions...
	imexam (cimg, 1, cimg, logfile=tmpctr, keeplog=yes, defkey="a",
		autoredraw=yes, allframes=no, nframes=0, ncstat=cbox,
		nlstat=cbox, graphcur="", imagecur=tmpcoo,
		wcs="logical", use_display=no, >& "dev$null")

	# Delete the input file if 'outab'=='intab' ...
	if ( ctr == coo ) { delete (coo, yes, verify=no) }

	# Extract the fitted center positions from the log file ...
	if (verbose) { print ("Recentered coordinates:", ) }
	list = tmpctr
	i = 0
	while ( fscan(list,x,y) != EOF ) {
	   i += 1
	   if ( i >= 3 ) {
	      xcen = x
	      ycen = y
	      print (x,y, >> ctr)
	      if (verbose) {
		 printf("#%d  xcen=%.3f, ycen=%.3f\n", (i-2), x, y)
	      }
	   }
	}

	# Clean-up ...
	delete (tmpcoo//","//tmpctr, yes, verify-, default_acti+)

end
