procedure imregister (input, output, refimage)

string	input	=""		{prompt="Input image"}
string	output	=""		{prompt="Output image"}
string	refimage=""		{prompt="Reference image"}
string	coords	=".coo"		{prompt="Reference coordinates file"}
int	cntrbox	=7		{prompt="Size of the fine centering box"}
int	srchbox	=11		{prompt="Size of the coarse centering box"}
bool	refrecen=no		{prompt="Recenter reference coordinates?"}
bool	execute	=yes		{prompt="Perform the image transformation?"}
string	interpol="spline3"	{prompt="Interpolation type (near|lin|poly3|poly5|spline3|sinc)",
				enum="nearest|linear|poly3|poly5|spline3|sinc"}
string	boundary="wrap"		{prompt="Boundary type (near|reflect|wrap)",
				enum="nearest|reflect|wrap"}
string	geometry="rscale"	{prompt="Geometric model (rscale|shift|xyscale|rotate|rxyscale|general)",
				enum="shift|xyscale|rotate|rscale|rxyscale|general"}
real	z1	=INDEF		{prompt="Lower cut level display"}
real	z2	=INDEF		{prompt="Upper cut level display"}
bool	verbose	=yes		{prompt="Print centers, and shifts?"}
string	logfile ="register.log"	{prompt="Text logfile name"}

# @(#) task imregister                Author: R.A. Jansen -- Jun 19 2000
# @(#)
# @(#) Task to register (shift, (de)magnify and rotate) an image using a
# @(#) list of matched input coordinates.  The coordinate file  need not
# @(#) yet exist or even be specified; if absent, the reference image is
# @(#) displayed and the user may interactively mark reference objects.
# @(#) Then the image that is to be registered to the reference image is
# @(#) displayed  with the numbered  positions  of the reference objects
# @(#) indicated.  If the only transformation required is a translation,
# @(#) then the user need only mark the position of  the first reference
# @(#) object and the remaining objects will be found automatically;  if
# @(#) there is also a magnification and/or rotation, then all objects
# @(#) need to be marked in the same order that the reference ones were
# @(#) marked.
# @(#) If a very large transformation is required, one may want to start
# @(#) with marking  only a few  reference objects  that can easilly  be
# @(#) identified in the image to be registered. And use more objects to
# @(#) perform the final registration during a second call to this task.
# @(#) If a tranposition is involved you may want to transpose the image
# @(#) first,  such that the image orientation is closer  to that of the
# @(#) reference image.
# @(#) Note, that interpolation of pixel values always leads to a slight
# @(#) loss in spatial resolution, and in a change of  the noise charac-
# @(#) teristics of the images (correlated noise).
# @(#)
# @(#) Uses 'tvmark','geomap','geotran' and several 'rjtools' tasks.
# @(#)
# @(#) Jul 12 2003 -- imroot->impath//imroot after change in chkimg. RAJ.
# @(#) Jul 29 2003 -- cparse->ocparse after revamp of cparse. RAJ.
# @(#) May 26 2007 -- added 'geometry' parameter (see manual of 'geomap'
# @(#)                on the meaning of the available choices). 'rscale'
# @(#)                used to be hardcoded and remains the default. Also
# @(#)                added cut level parameters 'z1' and 'z2'.  Default
# @(#)                values INDEF  ensure meaningful cut levels in most
# @(#)                cases, and allow input and reference image to have
# @(#)                different signal levels. RAJ.

begin
	string	img, refimg, coofil, outimg, intpl, bndry, fgeom, txtlog
	string	tmptvmark, tmpgeo, tmpsol, cstr
	int	cbox, sbox, ncol, nrow, NN, N1, N2
	real	x, y, x1, y1, x2, y2, zlo, zhi
	bool	oldcoo

	img	= input
	refimg	= refimage
	outimg	= output
	coofil	= coords
	if ( coofil == "" || coofil == " " ) { coofil = ".coo" }
	cbox	= cntrbox
	sbox	= srchbox
	intpl	= interpol
	bndry	= boundary
	fgeom	= geometry
	if ( fgeom == "" || fgeom == " " ) { fgeom = "rscale" }
	zlo	= z1
	zhi	= z2
	txtlog	= logfile

	# Check whether the required packages are loaded...
	if ( !defpac("tv") ) { tv }
	if ( !defpac("images") )  { image }
	if ( !defpac("immatch") ) { immatch }

	# Check existence of input, reference and output images...
	chkimg (img, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	img = chkimg.impath//chkimg.imroot//chkimg.imtype//chkimg.imrpstr

	chkimg (outimg, "noaccess", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	outimg = chkimg.impath//chkimg.imroot//chkimg.imtype//chkimg.imrpstr

	chkimg (refimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	refimg = chkimg.impath//chkimg.imroot//chkimg.imtype//chkimg.imrpstr

	if ( substr(coofil,1,1) == "." ) {
	   coofil = chkimg.impath//chkimg.imroot//coofil
	}

	# Determine the size of reference image ...
	hselect (refimg, "naxis1", yes ) | scan(ncol)
	hselect (refimg, "naxis2", yes ) | scan(nrow)

	# Opening message...
	gdate() ; sysinfo()
	if (verbose) { 
	   print ("IMREGISTER: input     = \""//img//"\"")
	   print ("            reference = \""//refimg//"\"")
	}
	print (" ", >> txtlog)
	print ("IMREGISTER:  "//sysinfo.sysmsg//"  "//gdate.tdate, >> txtlog)
	print ("    reference = "//refimg, >> txtlog)

	# Check whether existing coordinate file is supplied ...
	chkimg (coofil, "access", imtype="file", verbose=no)
	coofil = chkimg.impath//chkimg.imroot
	oldcoo = yes

	if ( !chkimg.ok) {
	   # Coordinate file does not yet exist; create one using 'tvmark'.
	   oldcoo = no

	   # Display the reference image ...
	   xdisplay (refimg, buff=1, z1=zlo, z2=zhi, lsig=3., hsig=15.,
		niter=1, zmode="linear", verbose=verbose)

	   # Let user mark reference objects interactively and write 'coofil'...
	   print ("Wait for the cursor cross to appear in the image display area, then")
	   print ("mark at least 3 objects (stars) for coordinate referencing using")
	   print ("the 'c' key (press q to quit).")
	   sleep(1)
	   tmptvmark = mktemp("tvmark")
	   tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		outimage="", mark="circle", radii=str(sbox), lengths="0",
		label=no, number=no, nxoffset=9, nyoffset=0, txsize=1,
		pointsize=3, font="raster", color=204, commands="",
		interactive=yes, > "dev$null", >& "STDERR" )

	   # Write the coordinate file
	   list = tmptvmark
	   while( fscan(list,x,y) != EOF ) {
	      if ( x < 1. )  x = 1.
	      if ( x > ncol) x = ncol
	      if ( y < 1. )  y = 1.
	      if ( y > nrow) y = nrow
	      print (x,y, >> coofil)
	   }
	   delete (tmptvmark, yes, verify=no)

	}

	if (refrecen) {
	   print ("Recentering objects in the reference coordinate list ...")
	   ctrcoo (coofil, coofil, refimg, cntrbox=cbox, redispl=no,
		   verbose=no)
	}
	iwc (coofil) ; N1 = iwc.nlines
	rdlist (coofil, 1)
	ocparse (rdlist.cline, delim=" ")
	x1 = real(ocparse.field1)
	y1 = real(ocparse.field2)


	# If a new reference coordinate file was created, save it...
	if (!oldcoo && coofil != chkimg.imroot ) {
	   chkimg (refimg, "access", verbose=no)
	   if (substr(chkimg.imrpstr,3,3) == "]") {
	      copy (coofil, chkimg.imroot//"."//substr(chkimg.imrpstr,2,2)//".coo", verbose=no)
	      print ("    refcoords = "//chkimg.imroot//"."//substr(chkimg.imrpstr,2,2)//".coo", >> txtlog)
	   } else {
	      copy (coofil, chkimg.imroot//".coo", verbose=no)
	      print ("    refcoords = "//chkimg.imroot//".coo", >> txtlog)
	   }
	} else {
	   print ("    refcoords = "//coofil, >> txtlog)
	}
	print ("    input     = "//img, >> txtlog)
	   
	# Display the image to be registered, and overlay the positions
	# of the stars in the reference frame ...
	xdisplay (img, buff=1, z1=zlo, z2=zhi, lsig=3., hsig=15., niter=1,
		zmode="linear", verbose=verbose)
	tvmark (frame=1, coor=coofil, logfile="", autolog=yes,
                outimage="", mark="plus", radii="0", lengths="0",
                label=no, number=yes, nxoffset=9, nyoffset=0, txsize=1,
                pointsize=3, font="raster", color=207, commands="",
                interactive=no, > "dev$null", >& "STDERR" )

	# Ask the user to mark all the stars ...
	print ("Wait for the cursor cross to appear in the image display area, then")
	print ("mark all reference stars in the same order using the 'c' key.")
        sleep(1)
	tmptvmark = mktemp("tvmark")
	tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
                outimage="", mark="circle", radii=str(cbox), lengths="0",
                label=no, number=no, nxoffset=9, nyoffset=0, txsize=1,
                pointsize=3, font="raster", color=204, commands="",
                interactive=yes, > "dev$null", >& "STDERR" )

	# Check whether all objects in the list were marked or whether
	# the positions in the reference image may serve as an initial
	# guess after taking into account a possible shift.
	iwc (tmptvmark) ; N2 = iwc.nlines
	rdlist (tmptvmark, 1)
	ocparse (rdlist.cline, delim=" ")
	x2 = real(ocparse.field1)
	y2 = real(ocparse.field2)
	if ( N2 > N1 ) { 
	   print ("ERROR: More objects marked than read from reference list!", >> txtlog)
	   error (1,"More objects marked than read from reference list!")
	}
	if ( N2 < N1 ) {
	   print ("Guessing the positions of the unmarked reference objects...")
	   for ( NN = (N2+1) ; NN <= N1; NN = NN + 1 ) {
	      rdlist (coofil, NN)
	      ocparse (rdlist.cline, delim=" ")
	      print ((real(ocparse.field1)+x2-x1),(real(ocparse.field2)+y2-y1), >> tmptvmark)
	   }
	}
	print ("TVMARK:  N_obj = "//str(N1)//",  N_marked = "//str(N2)//",  N_guessed = "//str(N1-N2), >> txtlog)

	# Show what we're doing...
	print ("Recentering objects in the coordinate list ...")
	ctrcoo (tmptvmark, tmptvmark, img, cntrbox=cbox, redispl=no,
		verbose=no)

	# Preparing the input coordinate file for 'geomap' ...
	tmpgeo = mktemp("tmpgeo_in")
	for ( NN = 1 ; NN < N1 ; NN = NN + 1 ) {
	    rdlist (coofil, NN) ; cstr = rdlist.cline
	    rdlist (tmptvmark, NN)
	    print (cstr//" "//rdlist.cline, >> tmpgeo)
	}

	# Derive the transformation (involving shifts, scaling and/or
	# image rotation.  Note that ONLY if 'fgeom'=="general", parameters
	# xxorder, xyorder, xxterms, yxorder, yyorder and yxterms are used;
	# they are ignored otherwise.  Hence, although the default is
	# 'rscale', we will set their values to be appropriate for 'general'.
	# If severely non-linear distortions need to be fit, then the order
	# may need to be increased from 3 to 4 or even higher.
	print ("GEOMAP:  Computing the spatial transformation function...")
	print ("GEOMAP:  Computing the spatial transformation function...", >> txtlog)
	if (access("geomap.sol")) { delete ("geomap.sol", verify=no) }
	tmpsol = mktemp("tmpgeo_sol")
	geomap (tmpgeo, "geomap.sol", 1., real(ncol), 1., real(nrow),
		transforms="", results="", fitgeometry=fgeom,
		function="polynomial", xxorder=3, xyorder=3, xxterms="half",
		yxorder=3, yyorder=3, yxterms="half", maxiter=1, reject=4.,
		calctype="real", verbose=verbose, interactive=no, graphics="",
		cursor="", > tmpsol)
	tail (tmpsol, nlines=8) | match ("Coordinate", stop=yes, >> txtlog)
	if (verbose) { tail (tmpsol, nlines=8) | match ("Coordinate", stop+) }

	delete (tmpsol, yes, verify=no)

	if (execute) {
	   print ("GEOTRAN: "//img//" --> "//outimg)
	   print ("GEOTRAN: interpol  = "//intpl//"   boundary="//bndry, >> txtlog)
	   print ("    "//img//" --> "//outimg, >> txtlog)
	   if ( fgeom == "general" ) {
	      geotran (img, outimg, "geomap.sol", tmpgeo, geometry="geometric",
		xin=INDEF, yin=INDEF, xshift=INDEF, yshift=INDEF, xout=INDEF,
		yout=INDEF, xmag=INDEF, ymag=INDEF, xrotation=INDEF,
		yrotation=INDEF, xmin=INDEF, xmax=INDEF, ymin=INDEF, ymax=INDEF,
		xscale=1., yscale=1., ncols=INDEF, nlines=INDEF, xsample=1.,
		ysample=1., interpolant=intpl, boundary=bndry, constant=0.,
		fluxconserve=yes, nxblock=512, nyblock=512, verbose=verbose)
	   } else {
	      geotran (img, outimg, "geomap.sol", tmpgeo, geometry="linear",
		xin=INDEF, yin=INDEF, xshift=INDEF, yshift=INDEF, xout=INDEF,
		yout=INDEF, xmag=INDEF, ymag=INDEF, xrotation=INDEF,
		yrotation=INDEF, xmin=INDEF, xmax=INDEF, ymin=INDEF, ymax=INDEF,
		xscale=1., yscale=1., ncols=INDEF, nlines=INDEF, xsample=1.,
		ysample=1., interpolant=intpl, boundary=bndry, constant=0.,
		fluxconserve=yes, nxblock=512, nyblock=512, verbose=verbose)
	   }

	   if (access("geomap.sol")) { delete ("geomap.sol", verify=no) }
	}

	if (access(tmptvmark))	{ delete (tmptvmark, yes, verify=no) }
	if (access(tmpgeo))	{ delete (tmpgeo, yes, verify=no) }

	coords = coofil

	if (verbose) { print ("Finished.") }
	print ("IMREGISTER: Finished.", >> txtlog)

end

