procedure markstds (image)

string	image	=""		{prompt="Image name"}
real	z1	=INDEF		{prompt="Lower cut level"}
real	z2	=INDEF		{prompt="Upper cut level"}
bool	redispl	=yes		{prompt="(Re)display image first?"}
string	radstr	="11,50,60"	{prompt="Radius of aperture and sky annulus"}
bool	append	=no		{prompt="Append to existing position file?"}
string	refstar	=""		{prompt="Optional name of reference star"}
bool	verbose	=yes		{prompt="Verbose messages?"}

# @(#) task markstds                  Author: R.A. Jansen -- May  2 1996
# @(#)
# @(#) task to interactively mark all standard stars in an image and log
# @(#) their positions to a file named 'image//".stdcoo".  The positions
# @(#) will have a label appended in column three.
# @(#) Parameter 'radstr' is a comma-separated list of the radius in pix
# @(#) corresponding to 7\" (Landolts standard measurement aperture) and
# @(#) the inner and outer radius of the circular annulus to measure the
# @(#) sky background.
# @(#) If a name is specified  for the reference star in the field, then
# @(#) it will be used; otherwise the image root name will be used.
# @(#)
# @(#) Calls 'rjtools' tasks: 'chkimg','sysinfo','gdate', and 'xdisplay'
# @(#)
# @(#) Jul.25 2002 -- Added parameters 'radstr', 'append' and 'redispl'.
# @(#)                Added support for other image formats. RAJ.
# @(#) Mar 27 2005 -- Added parameter 'refstar', cleaned-up code to in-
# @(#)                corporate it into 'rjtools_2.12' and wrote a help
# @(#)                file. RAJ.

begin
	string	cimg, cimr, crad, cnam, sfil, key, cmd, arg, val
	file	tmptvmark
	real	zlo, zhi, x, y
	int	nc, nr, ns, l

#	Initialize variables...
	cimg = image
	zlo  = z1
	zhi  = z2
	crad = radstr
	cnam = refstar

#	Check whether tv package is loaded...
	if ( !defpac("images") ) { images }
	if ( !defpac("tv") ) { tv }

#       Check whether input image exists
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	cimr	= chkimg.imroot
	cimg	= chkimg.impath//cimr//chkimg.imtype//chkimg.imrpstr
	if ( cnam == "" || cnam == " " ) cnam = cimr

#	Construct file name for star coordinates...
	sfil	= chkimg.impath//cimr//".stdcoo"
	
#	Determine input image size
	hselect (cimg, "i_naxis1", "yes") | scan (nc)
	hselect (cimg, "i_naxis2", "yes") | scan (nr)

#	Check that radii string is not empty and has proper format...
	if ( crad == "" || crad == " " ) {
	   print ("ERROR: 'radstr'==\"\" (empty string)! Specify radius in pixels corresponding")
	   print ("       to 7\" and inner and outer radius of the sky annulus, separated")
	   print ("       by commas.")
	   return
	}
	cparse (crad, delim=",", lignore+, mignore+, verbose-)
	if ( cparse.nfields != 3 ) {
	   printf("ERROR: 'radstr' has %d fields instead of 3! Specify radius in pixels\n", cparse.nfields)
	   print ("       corresponding to 7\" and inner and outer radius of the sky annulus,")
	   print ("       separated by commas.")
	   return
	}

#	Opening message...
	if (verbose) {
	   sysinfo() ; gdate(utc=no)
	   printf("MARKSTDS:   NOAO/"//sysinfo.sysmsg//"   "//gdate.tdate//"\n")
	   printf("   image = "//cimg//" [%d x %d]\n", nc, nr)
	}

#	If older version(s) of the star positions file are present and
#	'append=no', rename them.  The 'verbose' option in task 'copy'
#	seems not to be implemented!
	if ( access(sfil) && !append ) {
	   if ( access(sfil//"~") ) delete (sfil//"~", yes, verify-)
	   print ("Saving existing file \""//sfil//"\" --> \""//sfil//"~\"...")
	   rename (sfil, sfil//"~", field="all")
	}

#	(Re)display image if 'redispl=yes'...
	if (redispl) {
	   xdisplay (cimg, buff=1, z1=zlo, z2=zhi, lsig=3., hsig=15.,
		niter=1, zmode="linear", verbose=no)
	}

#       Mark all standard stars in the image...
	if ( append && access(sfil) ) {
	   print ("Appending positions to file \""//sfil//"\"...")
	}
	if (verbose) {
	   print ("Wait for the cursor cross to appear in the image display area, then mark")
	   print ("all (standard) stars in the image using the 'c' key.  Hit 'q' to quit.")
	   print ("Note: start with the main (i.e., naming or reference) star in the field.")
	} else {
	   print ("Mark all standard stars in the image using the 'c' key. Hit 'q' to quit.")
	}
	sleep(1)

	tmptvmark=mktemp("tvmark")

	tvmark (frame=1, coor="", logfile=tmptvmark, autolog=yes,
		outimage="", deletions="", commands="", mark="plus",
		radii="1", lengths="0", font="raster", color=204,
		label-, number-, nxoffset=0, nyoffset=0, pointsize=3,
		txsize=1, tolerance=1.5, inter+, > "dev$null", >& "STDERR")

	list = tmptvmark 
	ns = 0
	while( fscan(list,x,y,key,cmd,arg,val) != EOF ) {
	   if ( x < 1. ) x = 1.
	   if ( x > nc ) x = nc
	   if ( y < 1. ) y = 1.
	   if ( y > nr ) y = nr
	   if ( ns == 0 ) {
	      # Main (naming/reference) star in field
	      print (x,y,"\""//cnam//"\"", >> sfil)
	   } else {
	      print (x,y,"\""//str(ns)//"\"", >> sfil)
	   }
	   ns += 1
	}
	delete (tmptvmark, yes, verify-)

#	Mark all apertures in the image and overlay the sky annuli too...
	tvmark (1, coords=sfil, logfile="", autolog=no, outimage="",
		deletions="", commands="", mark="circle", radii=crad,
		lengths="0", font="raster", color=205, label=yes, number=no,
		nxoffset=16, nyoffset=16, pointsize=3, txsize=2,
		tolerance=1.5, interactive=no)

	list = ""
	print ("\nMARKSTDS: Finished.")

end
