procedure tvmarkall (image)

string	image	=""		{prompt="Image name"}
real	z1	=INDEF		{prompt="Lower cut level"}
real	z2	=INDEF		{prompt="Upper cut level"}
bool	redispl	=yes		{prompt="(Re)display image first?"}
bool	gal	=yes		{prompt="Mark galaxies?"}
bool	star	=yes		{prompt="Mark stars?"}
bool	cosmic	=yes		{prompt="Mark cosmics?"}
bool	del	=yes		{prompt="Mark deletions?"}
bool	region	=yes		{prompt="Mark regions?"}
string	regfil	=".reg"		{prompt="Name or extension of regions file"}
int	regclr	=212		{prompt="Color to use for regions"}
bool	dotransp=no		{prompt="transpose coordinates in mark files?"}

# @(#) task tvmarkall       Author: R.A. Jansen/S.P.W.(?) -- May 25 1994
# @(#)
# @(#) task to mark positions/regions on a loaded image.  Positions and/
# @(#) or regions are read  from files  'galfile', 'starfile', 'crfile',
# @(#) and 'delfile',  whose names are formed from 'image'  by appending
# @(#) '.pos.gal','.pos.star', '.pos.cosmic' and '.pos.del' to the image
# @(#) name. Each type of regions is overlayed on the image display in a
# @(#) different color. This task expands on an earlier script by SPW(?).
# @(#)
# @(#) Calls 'rjtools' tasks:  'chkimg' and 'xdisplay'
# @(#)
# @(#) Apr 19 1996 -- Added checks for existence region files:  ungrace-
# @(#)                full exits are fixed.  Added cut levels  for image
# @(#)                display. RAJ.
# @(#) Aug 21 1996 -- Added switches for marking  galaxy, stars, cosmics
# @(#)                and deletions separately;  added redisplay switch.
# @(#) Aug 23 1999 -- Added dotransp switch to allow use of same regions
# @(#)                files before and after transposing an image. RAJ.
# @(#) Jun 21 2000 -- Added support for multi-extension/N-dim images.RAJ
# @(#) Jul 27 2001 -- Added region switch to mark any region. RAJ.
# @(#) Aug 16 2003 -- Added support for commented lines (starting with a
# @(#)                '#'); cleaned-up code and wrote help file. RAJ.
# @(#) Jul 23 2004 -- Generalized marking of galaxies:  both the old way
# @(#)                of marking galaxy with a "plus" and marking by box
# @(#)                or circles is now supported. RAJ+AJM.

begin
	string	cimg, cimr, gfil, sfil, cfil, dfil, rfil, rest, cx, cr
	file	tmptvmark
	int	rclr, nc, nr
	real	zlo, zhi, x, y, r, tx, ty, tr, tt
	bool	gok, sok, cok, dok, rok

#	Initialize variables...
	cimg  = image
	zlo   = z1
	zhi   = z2
	rfil  = regfil
	rclr  = regclr
	if ( rclr == 0 ) rclr = 212

#	Check whether tv package is loaded...
	if ( !defpac("images") ) { images }
	if ( !defpac("tv") ) { tv }

#	Check whether image exists...
	chkimg (cimg, "access", imtype="", verbose=yes)
	if (!chkimg.ok) { return }
	cimr = chkimg.impath//chkimg.imroot
	cimg = cimr//chkimg.imtype//chkimg.imrpstr
	
#	Construct the region file names ...
	gfil = cimr//".pos.gal"
	sfil = cimr//".pos.star"
	cfil = cimr//".pos.cosmic"
	dfil = cimr//".pos.del"
	if ( substr(rfil,1,1) == "." ) rfil = cimr//rfil

#	Determine the image size...
	hselect (cimg, "i_naxis1", "yes") | scan (nc)
	hselect (cimg, "i_naxis2", "yes") | scan (nr)

#	(Re)display image if redispl=yes ...
	if (redispl) {
	   xdisplay (cimg, buff=1, z1=zlo, z2=zhi, lsig=3., hsig=15.,
		niter=1, zmode="linear", verbose=no)
        }

#       Check whether at least one positions/regions file is present...
	gok = no
	sok = no
	cok = no
	dok = no
	rok = no
	if ( !gal && !star && !cosmic && !del && !region ) {
	   error (1, "All 'Mark' switches are set to \"no\"!")
	}
	if (gal) {
	   chkimg (gfil, "access", imtype="file", verbose=no)
	   if (chkimg.ok) gok = yes
	}
	if (star) {
	   chkimg (sfil, "access", imtype="file", verbose=no)
	   if (chkimg.ok) sok = yes
	}
	if (cosmic) {
	   chkimg (cfil, "access", imtype="file", verbose=no)
	   if (chkimg.ok) cok = yes
	}
	if (del) {
	   chkimg (dfil, "access", imtype="file", verbose=no)
	   if (chkimg.ok) dok = yes
	}
	if (region) {
	   chkimg (rfil, "access", imtype="file", verbose=no)
	   if (!chkimg.ok) { return }
	   if ( rfil != "" ) rok = yes
	}
	if ( !gok && !sok && !cok && !dok && !rok ) {
	   error (1, "No positions/regions files found!")
	}

#	Apparently, at least one positions/regions file is present...
	# See if we need to mark galaxies...
	if ( gal && gok ) {
	   tmptvmark = mktemp("tvmark")
	   list = gfil

	   ## Original: (assumes that imput list contains only x,y)
	   #while ( fscan(list,rest) != EOF ) {
	   #   if ( substr(rest,1,1) != "#" ) print (rest, >> tmptvmark)
	   #}
	   #
	   #tvmark (frame=1, coor=tmptvmark, logfile="", autolog-, outimage="",
	   #	deletions="", commands="", mark="plus", radii="0",
	   #	lengths="0", font="raster", color=201, label=no, number=no,
	   #	nxoffset=0, nyoffset=0, pointsize=1, txsize=1, tolerance=1.5,
	   #	interactive=no, > "dev$null", >& "STDERR")

	   # New dd. Jul 23 2004 [RAJ+AJM]:
	   rest = ""
	   cr = ""
	   while( fscan(list,cx,y,cr,rest) != EOF ) {
	     if ( substr(cx,1,1) == '#' ) next
	     x = real(cx)
	     #print("DEBUG: cx=\""//cx//"\", y=",y,", cr=\""//cr//"\", rest=\""//rest//"\"")
	     if ( rest != "" ) {
		r = real(cr)
		if (dotransp) {
		   tx = nr - real(rest) + 1
		   ty = nr - r + 1
		   tr = nc - y + 1
		   tt = nc - x + 1
		   print (tx,tr," 201  b", >> tmptvmark)
		   print (ty,tt," 201  b", >> tmptvmark)
		} else {
		   print (x,r," 201  b", >> tmptvmark)
		   print (y,rest," 201  b", >> tmptvmark)
		}
	     } else {
		if ( cr != "" ) {
		   r = real(cr)
		   if (dotransp) {
		      tx = nr - y + 1
		      ty = nc - x + 1
		      print (tx,ty," 201  v", >> tmptvmark)
		      tx = tx + r
		      print (tx,ty," 201  v", >> tmptvmark)
		   } else {
		      print(x,y," 201  v", >> tmptvmark)
		      x = x + r
		      print(x,y," 201  v", >> tmptvmark)
		   }
		} else {
		   # original "xcen ycen" format: mark only center
		   if (dotransp) {
		      tx = nc - y + 1
		      ty = nr - x + 1
		      print (tx,(ty-2)," 201  b", >> tmptvmark)
		      print (tx,(ty+2)," 201  b", >> tmptvmark)
		      print ((tx-2),ty," 201  b", >> tmptvmark)
		      print ((tx+2),ty," 201  b", >> tmptvmark)
		   } else {
                      print (x,(y-2)," 201 b", >> tmptvmark)
		      print (x,(y+2)," 201 b", >> tmptvmark)
		      print ((x-2),y," 201 b", >> tmptvmark)
		      print ((x+2),y," 201 b", >> tmptvmark)
		   }
	        }
	     }
	     rest = ""
	     cr = ""
	   }

	   tvmark (frame=1, coor="", logfile="", autolog=no, outimage="",
		deletions="", commands=tmptvmark, mark="point", radii="0",
		lengths="0", font="raster", color=201, label=no, number=no,
		nxoffset=0, nyoffset=0, pointsize=3, txsize=1, tolerance=1.5,
		interactive=no, > "dev$null", >& "STDERR")

	   sleep(1)
	   delete (tmptvmark, yes, verify-)
	}

	# See if we need to mark stars...
	if ( star && sok ) {
	   tmptvmark = mktemp("tvmark")
	   list = sfil
	   rest = ""
	   while( fscan(list,cx,y,r,rest) != EOF ) {
	     if ( substr(cx,1,1) == '#' ) next
	     x = real(cx)
	     if ( rest != "" ) {
		if (dotransp) {
		   tx = nr - real(rest) + 1
		   ty = nr - r + 1
		   tr = nc - y + 1
		   tt = nc - x + 1
		   print (tx,tr," 204  b", >> tmptvmark)
		   print (ty,tt," 204  b", >> tmptvmark)
		} else {
		   print (x,r," 204  b", >> tmptvmark)
		   print (y,rest," 204  b", >> tmptvmark)
		}
	     } else {
		if (dotransp) {
		   tx = nr - y + 1
		   ty = nc - x + 1
		   print (tx,ty," 204  v", >> tmptvmark)
		   tx = tx + r
		   print (tx,ty," 204  v", >> tmptvmark)
		} else {
		   print(x,y," 204  v", >> tmptvmark)
		   x = x + r
		   print(x,y," 204  v", >> tmptvmark)
		}
	     }
	     rest = ""
	   }

	   tvmark (frame=1, coor="", logfile="", autolog=no, outimage="",
		deletions="", commands=tmptvmark, mark="point", radii="0",
		lengths="0", font="raster", color=204, label=no, number=no,
		nxoffset=0, nyoffset=0, pointsize=3, txsize=1, tolerance=1.5,
		interactive=no, > "dev$null", >& "STDERR")

	   sleep (1)
	   delete (tmptvmark, yes, verify-)
        }

	# See if we need to mark defects or deleted regions...
	if ( del && dok ) {
	   tmptvmark = mktemp("tvmark")
	   list = dfil
	   rest = ""
	   while( fscan(list,cx,y,r,rest) != EOF ) {
	     if ( substr(cx,1,1) == '#' ) next
	     x = real(cx)
	     if ( rest != "" ) {
		if (dotransp) {
		   tx = nr - real(rest) + 1
		   ty = nr - r + 1
		   tr = nc - y + 1
		   tt = nc - x + 1
		   print (tx,tr," 207  b", >> tmptvmark)
		   print (ty,tt," 207  b", >> tmptvmark)
		} else {
		   print (x,r," 207  b", >> tmptvmark)
		   print (y,rest," 207  b", >> tmptvmark)
		}
	     } else {
		if (dotransp) {
		   tx = nr - y + 1
		   ty = nc - x + 1
		   print (tx,ty," 207  v", >> tmptvmark)
		   tx = tx + r
		   print (tx,ty," 207  v", >> tmptvmark)		   
		} else {
		   print (x,y," 207  v", >> tmptvmark)
		   x = x + r
		   print (x,y," 207  v", >> tmptvmark)
		}
	     }
	     rest = ""
	   }

	   tvmark (frame=1, coor="", logfile="", autolog=no, outimage="",
		deletions="", commands=tmptvmark, mark="point", radii="0",
		lengths="0", font="raster", color=207, label=no, number=no,
		nxoffset=0, nyoffset=0, pointsize=3, txsize=1, tolerance=1.5,
		interactive=no, > "dev$null", >& "STDERR")

	   sleep (1)
	   delete (tmptvmark, yes, verify-)
	}

	# See if we need to mark cosmic ray hits...
	if ( cosmic && cok ) {
	   tmptvmark = mktemp("tvmark")
	   list = cfil
	   rest = ""
	   while( fscan(list,cx,y,r,rest) != EOF ) {
	     if ( substr(cx,1,1) == '#' ) next
	     x = real(cx)
	     if ( rest != "" ) {
		if (dotransp) {
		   tx = nr - real(rest) + 1
		   ty = nr - r + 1
		   tr = nc - y + 1
		   tt = nc - x + 1
		   print (tx,tr," 206  b", >> tmptvmark)
		   print (ty,tt," 206  b", >> tmptvmark)
		} else {
		   print (x,r," 206  b", >> tmptvmark)
		   print (y,rest," 206  b", >> tmptvmark)
		}
	     } else {
		if (dotransp) {
		   tx = nr - y + 1
		   ty = nc - x + 1
		   print (tx,ty," 206  v", >> tmptvmark)
		   tx = tx + r
		   print (tx,ty," 206  v", >> tmptvmark)		   
		} else {
		   print (x,y," 206  v", >> tmptvmark)
		   x = x + r
		   print (x,y," 206  v", >> tmptvmark)
		}
	     }
	     rest = ""
	   }

	   tvmark (frame=1, coor="", logfile="", autolog=no, outimage="",
		deletions="", commands=tmptvmark, mark="point", radii="0",
		lengths="0", font="raster", color=206, label=no, number=no,
		nxoffset=0, nyoffset=0, pointsize=3, txsize=1, tolerance=1.5,
		interactive=no, > "dev$null", >& "STDERR")

	   sleep (1)
	   delete (tmptvmark, yes, verify-)
	}

	# See if we need to mark regions (of arbitrary significance)...
	if ( region && rok ) {
	   tmptvmark = mktemp("tvmark")
	   list = rfil
	   rest = ""
	   while ( fscan(list,cx,y,r,rest) != EOF ) {
	     if ( substr(cx,1,1) == '#' ) next
	     x = real(cx)
	     if ( rest != "" ) {
		if (dotransp) {
		   tx = nr - real(rest) + 1
		   ty = nr - r + 1
		   tr = nc - y + 1
		   tt = nc - x + 1
		   print (tx,tr," ",rclr," b", >> tmptvmark)
		   print (ty,tt," ",rclr," b", >> tmptvmark)
		} else {
		   print (x,r," ",rclr," b", >> tmptvmark)
		   print (y,rest," ",rclr," b", >> tmptvmark)
		}
	     } else {
		if (dotransp) {
		   tx = nr - y + 1
		   ty = nc - x + 1
		   print (tx,ty," ",rclr," v", >> tmptvmark)
		   tx = tx + r
		   print (tx,ty," ",rclr," v", >> tmptvmark)
		} else {
		   print (x,y," ",rclr," v", >> tmptvmark)
		   x = x + r
		   print (x,y," ",rclr," v", >> tmptvmark)
		}
	     }
	     rest = ""
	   }

	   tvmark (frame=1, coor="", logfile="", autolog=no, outimage="",
		deletions="", commands=tmptvmark, mark="point", radii="0",
		lengths="0", font="raster", color=rclr, label=no, number=no,
		nxoffset=0, nyoffset=0, pointsize=3, txsize=1, tolerance=1.5,
		interactive=no, > "dev$null", >& "STDERR")

	   sleep (1)
	   delete (tmptvmark, yes, verify-)
	}
         
end
